package com.terraformersmc.terrestria;

import com.terraformersmc.terraform.block.TerraformSignBlock;
import com.terraformersmc.terraform.entity.TerraformBoatEntity;
import com.terraformersmc.terraform.registry.SpriteIdentifierRegistry;
import com.terraformersmc.terrestria.init.TerrestriaBlocks;
import com.terraformersmc.terrestria.init.TerrestriaEntities;
import com.terraformersmc.terrestria.init.TerrestriaItems;
import net.fabricmc.api.ClientModInitializer;
import net.fabricmc.fabric.api.blockrenderlayer.v1.BlockRenderLayerMap;
import net.fabricmc.fabric.api.client.render.ColorProviderRegistry;
import net.fabricmc.fabric.api.client.rendereregistry.v1.EntityRendererRegistry;
import net.minecraft.class_1163;
import net.minecraft.class_1299;
import net.minecraft.class_1921;
import net.minecraft.class_1926;
import net.minecraft.class_1933;
import net.minecraft.class_2248;
import net.minecraft.class_2960;
import net.minecraft.class_322;
import net.minecraft.class_326;
import net.minecraft.class_4722;
import net.minecraft.class_4730;
import net.minecraft.class_881;

// This class is an entrypoint
@SuppressWarnings("unused")
public class TerrestriaClient implements ClientModInitializer {
	private static final class_1921 LEAVES_ITEM_LAYER = class_4722.method_24074();
	private static final class_1921 GRASS_BLOCK_LAYER = class_1921.method_23579();
	private static final class_1921 PLANT_BLOCK_LAYER = class_1921.method_23581();
	private static final class_1921 DOOR_BLOCK_LAYER = class_1921.method_23581();

	private static final class_322 FOLIAGE_BLOCK_COLORS =
			(block, world, pos, layer) -> world != null && pos != null ? class_1163.method_4966(world, pos) : class_1926.method_8341();
	private static final class_322 GRASS_BLOCK_COLORS =
			(block, world, pos, layer) -> world != null && pos != null ? class_1163.method_4962(world, pos) : class_1933.method_8377(0.5, 1.0);
	private static final class_326 FOLIAGE_ITEM_COLORS =
			(item, layer) -> class_1926.method_8344(0.5, 1.0);
	private static final class_326 GRASS_ITEM_COLORS =
			(item, layer) -> class_1933.method_8377(0.5, 1.0);

	@Override
	public void onInitializeClient() {
		ColorProviderRegistry.BLOCK.register(
				FOLIAGE_BLOCK_COLORS,
				TerrestriaBlocks.RUBBER.leaves,
				TerrestriaBlocks.CYPRESS.leaves,
				TerrestriaBlocks.WILLOW.leaves,
				TerrestriaBlocks.RAINBOW_EUCALYPTUS.leaves,
				TerrestriaBlocks.JAPANESE_MAPLE_SHRUB_LEAVES,
				TerrestriaBlocks.REDWOOD.leaves,
				TerrestriaBlocks.HEMLOCK.leaves
		);

		BlockRenderLayerMap.INSTANCE.putBlock(TerrestriaBlocks.SAKURA_LEAF_PILE, class_1921.method_23579());

		BlockRenderLayerMap.INSTANCE.putBlocks(
			DOOR_BLOCK_LAYER,
			TerrestriaBlocks.REDWOOD.door,
			TerrestriaBlocks.RAINBOW_EUCALYPTUS.door,
			TerrestriaBlocks.CYPRESS.door,
			TerrestriaBlocks.WILLOW.door,
			TerrestriaBlocks.JAPANESE_MAPLE.door,
			TerrestriaBlocks.RAINBOW_EUCALYPTUS.trapdoor,
			TerrestriaBlocks.CYPRESS.trapdoor,
			TerrestriaBlocks.WILLOW.trapdoor
		);

		BlockRenderLayerMap.INSTANCE.putBlocks(
				PLANT_BLOCK_LAYER,
				TerrestriaBlocks.REDWOOD_SAPLING,
				TerrestriaBlocks.HEMLOCK_SAPLING,
				TerrestriaBlocks.RUBBER_SAPLING,
				TerrestriaBlocks.CYPRESS_SAPLING,
				TerrestriaBlocks.WILLOW_SAPLING,
				TerrestriaBlocks.JAPANESE_MAPLE_SAPLING,
				TerrestriaBlocks.JAPANESE_MAPLE_SHRUB_SAPLING,
				TerrestriaBlocks.DARK_JAPANESE_MAPLE_SAPLING,
				TerrestriaBlocks.RAINBOW_EUCALYPTUS_SAPLING,
				TerrestriaBlocks.SAKURA_SAPLING,
				TerrestriaBlocks.JUNGLE_PALM_SAPLING,
				TerrestriaBlocks.CATTAIL,
				TerrestriaBlocks.TALL_CATTAIL,
				TerrestriaBlocks.INDIAN_PAINTBRUSH,
				TerrestriaBlocks.MONSTERAS
		);

		BlockRenderLayerMap.INSTANCE.putBlocks(
				PLANT_BLOCK_LAYER,
				TerrestriaBlocks.POTTED_REDWOOD_SAPLING,
				TerrestriaBlocks.POTTED_HEMLOCK_SAPLING,
				TerrestriaBlocks.POTTED_RUBBER_SAPLING,
				TerrestriaBlocks.POTTED_CYPRESS_SAPLING,
				TerrestriaBlocks.POTTED_WILLOW_SAPLING,
				TerrestriaBlocks.POTTED_JAPANESE_MAPLE_SAPLING,
				TerrestriaBlocks.POTTED_JAPANESE_MAPLE_SHRUB_SAPLING,
				TerrestriaBlocks.POTTED_DARK_JAPANESE_MAPLE_SAPLING,
				TerrestriaBlocks.POTTED_RAINBOW_EUCALYPTUS_SAPLING,
				TerrestriaBlocks.POTTED_SAKURA_SAPLING,
				TerrestriaBlocks.POTTED_JUNGLE_PALM_SAPLING,
				TerrestriaBlocks.POTTED_INDIAN_PAINTBRUSH,
				TerrestriaBlocks.POTTED_MONSTERAS
		);

		addColoredGrass(TerrestriaBlocks.BASALT_GRASS_BLOCK);

		addSigns(
				TerrestriaBlocks.REDWOOD.sign,
				TerrestriaBlocks.HEMLOCK.sign,
				TerrestriaBlocks.RUBBER.sign,
				TerrestriaBlocks.CYPRESS.sign,
				TerrestriaBlocks.WILLOW.sign,
				TerrestriaBlocks.JAPANESE_MAPLE.sign,
				TerrestriaBlocks.RAINBOW_EUCALYPTUS.sign,
				TerrestriaBlocks.SAKURA.sign
		);

		ColorProviderRegistry.ITEM.register(
				FOLIAGE_ITEM_COLORS,
				TerrestriaItems.RUBBER.leaves,
				TerrestriaItems.CYPRESS.leaves,
				TerrestriaItems.WILLOW.leaves,
				TerrestriaItems.RAINBOW_EUCALYPTUS.leaves,
				TerrestriaItems.JAPANESE_MAPLE_SHRUB_LEAVES,
				TerrestriaItems.REDWOOD.leaves,
				TerrestriaItems.HEMLOCK.leaves
		);

		BlockRenderLayerMap.INSTANCE.putItem(TerrestriaItems.SAKURA_LEAF_PILE, class_1921.method_23579());

		ColorProviderRegistry.ITEM.register(
				GRASS_ITEM_COLORS,
				TerrestriaItems.BASALT_GRASS_BLOCK
		);

		addBoatRenderer(TerrestriaEntities.REDWOOD_BOAT);
		addBoatRenderer(TerrestriaEntities.HEMLOCK_BOAT);
		addBoatRenderer(TerrestriaEntities.RUBBER_BOAT);
		addBoatRenderer(TerrestriaEntities.CYPRESS_BOAT);
		addBoatRenderer(TerrestriaEntities.WILLOW_BOAT);
		addBoatRenderer(TerrestriaEntities.JAPANESE_MAPLE_BOAT);
		addBoatRenderer(TerrestriaEntities.RAINBOW_EUCALYPTUS_BOAT);
		addBoatRenderer(TerrestriaEntities.SAKURA_BOAT);
	}

	private void addSigns(TerraformSignBlock... signs) {
		for(TerraformSignBlock sign: signs) {
			addSign(sign);
		}
	}

	private void addSign(TerraformSignBlock sign) {
		class_2960 texture = sign.getTexture();

		SpriteIdentifierRegistry.INSTANCE.addIdentifier(new class_4730(class_4722.field_21708, texture));
	}

	private void addColoredGrass(class_2248 grass) {
		BlockRenderLayerMap.INSTANCE.putBlock(grass, GRASS_BLOCK_LAYER);
		ColorProviderRegistry.BLOCK.register(GRASS_BLOCK_COLORS, grass);
	}

	private void addBoatRenderer(class_1299<TerraformBoatEntity> boat) {
		EntityRendererRegistry.INSTANCE.register(boat, (dispatcher, context) -> new class_881(dispatcher));
	}
}
