package com.terraformersmc.terrestria.feature.trees;

import com.mojang.datafixers.Dynamic;
import java.util.Random;
import java.util.Set;
import java.util.function.Function;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2680;
import net.minecraft.class_2944;
import net.minecraft.class_3341;
import net.minecraft.class_3481;
import net.minecraft.class_3746;
import net.minecraft.class_3747;
import net.minecraft.class_4640;

public class PalmTreeFeature extends class_2944<class_4640> {
	private class_2680 bark;

	public PalmTreeFeature(Function<Dynamic<?>, ? extends class_4640> function, class_2680 bark) {
		super(function);

		this.bark = bark;
	}

	private static class_2350 spiral(class_2350 direction, boolean invert) {
		switch (direction) {
			case field_11034:
				return invert ? class_2350.field_11043 : class_2350.field_11035;
			case field_11039:
				return invert ? class_2350.field_11035 : class_2350.field_11043;
			case field_11043:
				return invert ? class_2350.field_11039 : class_2350.field_11034;
			case field_11035:
			default:
				return invert ? class_2350.field_11034 : class_2350.field_11039;
		}
	}

	@Override
	public boolean generate(class_3747 world, Random rand, class_2338 origin, Set<class_2338> logs, Set<class_2338> leaves, class_3341 box, class_4640 config) {
		// Total trunk height
		int height = rand.nextInt(5) + 8;

		if (origin.method_10264() + height + 1 > 256 || origin.method_10264() < 1) {
			return false;
		}

		class_2338 below = origin.method_10074();

		boolean sand = false;

		if (!method_16430(world, below)) {
			if(world.method_16358(below, state -> state.method_11602(class_3481.field_15466))) {
				sand = true;
			} else {
				return false;
			}
		}

		if(!check(world, origin, height)) {
			return false;
		}

		if(!sand) {
			method_16427(world, below);
		}

		class_2338.class_2339 pos = new class_2338.class_2339(origin);
		growTrunk(world, rand, pos, logs, box, config, height);
		growLeaves(world, rand, pos, leaves, box, config);

		return true;
	}

	private boolean check(class_3746 world, class_2338 origin, int height) {
		class_2338.class_2339 pos = new class_2338.class_2339(origin);

		for(int y = 0; y < height; y++) {
			int radius = y >= 2 ? 1 : 0;

			for(int z = -radius; z <= radius; z++) {
				for(int x = -radius; x <= radius; x++) {
					pos.method_10101(origin).method_10100(x, y, z);

					if(!method_16432(world, pos)) {
						return false;
					}
				}
			}
		}

		return true;
	}

	// Grows the bent trunk of the tree.
	private void growTrunk(class_3747 world, Random rand, class_2338.class_2339 pos, Set<class_2338> logs, class_3341 box, class_4640 config, int height) {
		for (int i = 0; i < 2; i++) {
			method_23382(world, rand, pos, logs, box, config);
			pos.method_10098(class_2350.field_11036);
		}

		int run = 0;

		int velocityX = rand.nextBoolean() ? 1 : -1;
		int velocityZ = rand.nextBoolean() ? 1 : -1;

		for (int i = 2; i < height; i++) {
			if(!method_16432(world, pos)) {
				return;
			}

			if (run++ == 3) {
				PortUtil.setBlockState(logs, world, pos, bark, box);

				if (rand.nextBoolean()) {
					pos.method_10100(velocityX, 0, 0);
				} else {
					pos.method_10100(0, 0, velocityZ);
				}
				run = 0;

				if(!method_16432(world, pos)) {
					return;
				}

				PortUtil.setBlockState(logs, world, pos, bark, box);
			} else {
				method_23382(world, rand, pos, logs, box, config);
			}

			pos.method_10098(class_2350.field_11036);
		}

		method_23382(world, rand, pos, logs, box, config);
	}

	private void growLeaves(class_3747 world, Random rand, class_2338.class_2339 pos, Set<class_2338> leaves, class_3341 box, class_4640 config) {
		class_2338 center = pos.method_10062();

		method_23383(world, rand, pos.method_10101(center).method_10100(0, 1, 0), leaves, box, config);
		method_23383(world, rand, pos.method_10101(center).method_10100(1, 1, 0), leaves, box, config);
		method_23383(world, rand, pos.method_10101(center).method_10100(0, 1, 1), leaves, box, config);
		method_23383(world, rand, pos.method_10101(center).method_10100(-1, 1, 0), leaves, box, config);
		method_23383(world, rand, pos.method_10101(center).method_10100(0, 1, -1), leaves, box, config);

		boolean invertLeafSpiral = rand.nextBoolean();

		for (int dZ = -1; dZ < 2; dZ++) {
			for (int dX = -1; dX < 2; dX++) {
				method_23383(world, rand, pos.method_10101(center).method_10100(dZ, 0, dX), leaves, box, config);
			}
		}

		for (int d = 0; d < 4; d++) {
			class_2350 direction = class_2350.method_10139(d);

			pos.method_10101(center).method_10104(direction, 2);
			placeSpiral(world, rand, pos, leaves, box, config, direction, !invertLeafSpiral);

			pos.method_10101(center).method_10104(direction, 3);
			placeSpiral(world, rand, pos, leaves, box, config, direction, invertLeafSpiral);
		}
	}

	private void placeSpiral(class_3747 world, Random rand, class_2338.class_2339 pos, Set<class_2338> leaves, class_3341 box, class_4640 config, class_2350 direction, boolean invertLeafSpiral) {
		method_23383(world, rand, pos, leaves, box, config);

		class_2350 spiral = spiral(direction, invertLeafSpiral);
		method_23383(world, rand, pos.method_10098(spiral), leaves, box, config);

		for (int i = 0; i < 2; i++) {
			method_23383(world, rand, pos.method_10098(class_2350.field_11033), leaves, box, config);
		}
	}
}
