package com.terraformersmc.terrestria.feature.trees;

import com.mojang.datafixers.Dynamic;
import java.util.Random;
import java.util.Set;
import java.util.function.Function;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2944;
import net.minecraft.class_3341;
import net.minecraft.class_3532;
import net.minecraft.class_3746;
import net.minecraft.class_3747;
import net.minecraft.class_4640;

public class RubberTreeFeature extends class_2944<class_4640> {
	public RubberTreeFeature(Function<Dynamic<?>, ? extends class_4640> function) {
		super(function);
	}

	@Override
	public boolean generate(class_3747 world, Random rand, class_2338 origin, Set<class_2338> logs, Set<class_2338> leaves, class_3341 box, class_4640 config) {
		// Total trunk height
		int height = rand.nextInt(4) + 12;

		if (origin.method_10264() + height + 1 > 256 || origin.method_10264() < 1) {
			return false;
		}

		class_2338 below = origin.method_10074();

		if (!method_16430(world, below)) {
			return false;
		}

		if (!checkForObstructions(world, origin, height)) {
			return false;
		}

		method_16427(world, below);
		growTrunk(world, rand, new class_2338.class_2339(origin), logs, box, config, height);
		growBranches(world, rand, new class_2338.class_2339(origin), logs, leaves, box, config, height);

		return true;
	}

	private boolean checkForObstructions(class_3746 world, class_2338 origin, int height) {
		class_2338.class_2339 pos = new class_2338.class_2339(origin);

		for (int dY = 0; dY < height; dY++) {
			pos.method_10099(origin.method_10264() + dY);

			if (!method_16432(world, pos)) {
				return false;
			}
		}

		return true;
	}

	private void growTrunk(class_3747 world, Random rand, class_2338.class_2339 pos, Set<class_2338> logs, class_3341 box, class_4640 config, int height) {
		for (int i = 0; i < height; i++) {
			method_23382(world, rand, pos, logs, box, config);
			pos.method_10098(class_2350.field_11036);
		}
	}

	private void growBranches(class_3747 world, Random rand, class_2338.class_2339 pos, Set<class_2338> logs, Set<class_2338> leaves, class_3341 box, class_4640 config, int height) {
		int x = pos.method_10263();
		int y = pos.method_10264();
		int z = pos.method_10260();

		for (int branch = 0; branch < 16; branch++) {
			int baseY = rand.nextInt(height - 2) + 4;

			float length = rand.nextFloat() * 7 + 2;
			float angle = rand.nextFloat() * (float) Math.PI * 2;

			int offsetX = (int) (class_3532.method_15362(angle) * length);
			int offsetZ = (int) (class_3532.method_15374(angle) * length);

			int moveX = offsetX > 0 ? 1 : -1;
			int moveZ = offsetZ > 0 ? 1 : -1;

			int absX = Math.abs(offsetX);
			int absZ = Math.abs(offsetZ);

			int movedX = 0;
			int movedZ = 0;

			float stepX = 1.0F / absX;
			float stepZ = 1.0F / absZ;

			for (int movement = 0; movement < absX + absZ; movement++) {
				if (Math.abs(movedX * stepX) < Math.abs(movedZ * stepZ) && Math.abs(movedX) < absX) {
					movedX += moveX;
				} else {
					movedZ += moveZ;
				}

				int offsetY = (int) (Math.sqrt(movedX * movedX + movedZ * movedZ) * 0.4);

				if (movedX > 2 || movedX < -2 || movedZ > 2 || movedZ < -2) {
					continue;
				}

				pos.method_10103(x + movedX, y + baseY + offsetY, z + movedZ);

				if (!method_16432(world, pos)) {
					break;
				}

				method_23382(world, rand, pos, logs, box, config);

				for (class_2350 direction : class_2350.values()) {
					pos.method_10103(x + movedX, y + baseY + offsetY, z + movedZ);
					pos.method_10098(direction);

					method_23383(world, rand, pos, leaves, box, config);
				}
			}
		}
	}
}
