package com.terraformersmc.terrestria.feature.trees;

import com.mojang.datafixers.Dynamic;
import com.terraformersmc.terraform.util.*;
import java.util.Random;
import java.util.Set;
import java.util.function.Function;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2410;
import net.minecraft.class_2680;
import net.minecraft.class_2944;
import net.minecraft.class_3341;
import net.minecraft.class_3746;
import net.minecraft.class_3747;
import net.minecraft.class_4640;

public class WillowTreeFeature extends class_2944<class_4640> {
	public WillowTreeFeature(Function<Dynamic<?>, ? extends class_4640> function) {
		super(function);
	}

	@Override
	public boolean generate(class_3747 world, Random rand, class_2338 origin, Set<class_2338> logs, Set<class_2338> leaves, class_3341 box, class_4640 config) {
		// Total tree height
		int height = rand.nextInt(3) + 8;

		// Maximum leaf radius.
		double maxRadius = 5 + 3 * rand.nextDouble();

		// Minimum leaf radius
		double minRadius = 1.25 + 2 * rand.nextDouble();

		// If the tree can pass the max build height
		if (origin.method_10264() + height + 1 > 256 || origin.method_10264() < 1) {
			return false;
		}

		// Make sure the tree can grow where the origin is
		class_2338 below = origin.method_10074();

		if (!method_16430(world, below)) {
			return false;
		}
		if (!checkForObstructions(world, origin, height, (int) Math.ceil(maxRadius))) {
			return false;
		}

		method_16427(world, below);

		// Grow the trunk
		growTrunk(world, rand, new class_2338.class_2339(origin), logs, leaves, box, config, height);
		growBranches(world, rand, new class_2338.class_2339(origin.method_10079(class_2350.field_11036, height / 3)), logs, box, config, (int) maxRadius - 1);
		growBranches(world, rand, new class_2338.class_2339(origin.method_10079(class_2350.field_11036, (height / 2) + 1)), logs, box, config, (int) (maxRadius * radiusFactor(height / 2, height)));

		// Grow the leaves
		class_2338.class_2339 pos = new class_2338.class_2339(origin);
		growLeaves(world, rand, pos, leaves, box, config, height, maxRadius, minRadius);

		return true;
	}

	// Grows the center trunk.
	private void growTrunk(class_3747 world, Random rand, class_2338.class_2339 pos, Set<class_2338> logs, Set<class_2338> leaves, class_3341 box, class_4640 config, int height) {
		// Grows the trunk at 80% of it's total height (so the trunk doesn't poke out)
		for (int i = 0; i < (height * .8); i++) {
			method_23382(world, rand, pos, logs, box, config);
			pos.method_10098(class_2350.field_11036);
		}

		// Make sure there are leaf blocks on the top of the tree
		Shapes.circle(pos, 1.5, position -> {
			method_23383(world, rand, pos, leaves, box, config);
		});
	}

	private void growBranches(class_3747 world, Random rand, class_2338.class_2339 pos, Set<class_2338> logs, class_3341 box, class_4640 config, int maxRadius) {
		// Save original origin for use in z branch

		class_2338 origin = pos.method_10062();
		int branchLength = maxRadius - 2;

		pos.method_10104(class_2350.field_11039, branchLength);
		for (int x = -branchLength; x <= branchLength; x++) {
			if (method_16420(world, pos)) {
				class_2680 state = config.field_21288.method_23455(rand, pos);

				PortUtil.setBlockState(logs, world, pos, state.method_11657(class_2410.field_11459, class_2350.field_11039.method_10166()), box);
			}
			pos.method_10098(class_2350.field_11034);
		}

		pos.method_10101(origin).method_10104(class_2350.field_11043, branchLength);
		for (int z = -branchLength; z <= branchLength; z++) {
			if (method_16420(world, pos)) {
				class_2680 state = config.field_21288.method_23455(rand, pos);

				PortUtil.setBlockState(logs, world, pos, state.method_11657(class_2410.field_11459, class_2350.field_11043.method_10166()), box);
			}
			pos.method_10098(class_2350.field_11035);
		}

		// Reset the branches
		pos.method_10101(origin);
	}

	private void growDangingBit(class_3747 world, Random rand, class_2338.class_2339 pos, Set<class_2338> leaves, class_3341 box, class_4640 config) {
		int randHolder;

		// 33% chance of a dangling bit generating.
		if (rand.nextInt(3) == 1) {
			// Length of the bit

			randHolder = rand.nextInt(3);
			for (int d = 0; d < randHolder; d++) {
				pos.method_10098(class_2350.field_11033);
				method_23383(world, rand, pos, leaves, box, config);
			}

			pos.method_10104(class_2350.field_11036, randHolder);
		}
	}

	// ModifiableTestableWorld world, Random rand, BlockPos.Mutable pos, Set<BlockPos> leaves, BlockBox box, BranchedTreeFeatureConfig config

	private void growLeaves(class_3747 world, Random rand, class_2338.class_2339 pos, Set<class_2338> leaves, class_3341 box, class_4640 config, int height, double maxRadius, double minRadius) {
		int x = pos.method_10263();
		int y = pos.method_10264();
		int z = pos.method_10260();
		double radius;
		double innerRadius;

		for (int dy = 0; dy < height; dy++) {

			// If the leaves aren't 1/3 way up the trunk
			if (dy < (height / 3)) {
				continue;
			}

			pos.method_10103(x, y + dy, z);

			radius = maxRadius * radiusFactor(dy, height);
			innerRadius = minRadius * radiusFactor(dy, height);

			if (radius < 0) {
				continue;
			}

			int finalDy = dy;
			Shapes.canopyCircle(pos, radius, innerRadius, position -> {
				if (method_23383(world, rand, pos, leaves, box, config) && finalDy == (height / 3)) {
					growDangingBit(world, rand, pos, leaves, box, config);
				}
			});
		}
	}

	// Provides the factor to the radius, where x is a double from 0.0 to the height that represents the progress along the trunk.
	private double radiusFactor(double x, double height) {
		// Makes the polynomial apply to values from 0-the height
		x = x / height;

		// A 3rd-degree polynomial approximating the shape of a willow tree. from 0-1
		return 1.88 * (x * x * x) - 6.52 * (x * x) + 4.63 * x;
	}

	private boolean checkForObstructions(class_3746 world, class_2338 origin, int height, int radius) {
		class_2338.class_2339 pos = new class_2338.class_2339(origin);

		for (int dY = origin.method_10264(); dY < height; dY++) {
			for (int dZ = -radius; dZ <= radius; dZ++) {
				for (int dX = -radius; dX <= radius; dX++) {
					pos.method_10103(origin.method_10263() + dX, origin.method_10264() + dY, origin.method_10260() + dZ);

					if (!method_16432(world, pos)) {
						return false;
					}
				}
			}
		}

		pos.method_10103(origin.method_10263(), origin.method_10264() + height, origin.method_10260());

		for (int i = 0; i < 4; i++) {
			if (!method_16432(world, pos.method_10098(class_2350.field_11036))) {
				return false;
			}
		}

		return true;
	}
}
