package com.terraformersmc.terrestria.feature.trees.templates;

import com.mojang.datafixers.Dynamic;
import com.terraformersmc.terrestria.feature.trees.PortUtil;
import java.util.Random;
import java.util.Set;
import java.util.function.Function;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2944;
import net.minecraft.class_3341;
import net.minecraft.class_3746;
import net.minecraft.class_3747;
import net.minecraft.class_4640;

public class ConiferTreeFeature extends class_2944<class_4640> {
	public ConiferTreeFeature(Function<Dynamic<?>, ? extends class_4640> function) {
		super(function);
	}

	@Override
	public boolean generate(class_3747 world, Random rand, class_2338 origin, Set<class_2338> logs, Set<class_2338> leaves, class_3341 box, class_4640 config) {

		int height = getLeafHeight(rand);
		int bareTrunkHeight = getBareTrunkHeight(rand);
		int maxLeafRadius = getMaxLeafRadius(rand);

		if (origin.method_10264() + height + 1 > 256 || origin.method_10264() < 1) {
			return false;
		}

		class_2338 below = origin.method_10074();

		if (!method_16430(world, below)) {
			return false;
		}

		if (!checkForObstructions(world, origin, height, bareTrunkHeight, maxLeafRadius)) {
			return false;
		}

		method_16427(world, origin.method_10074());
		growTrunk(world, rand, new class_2338.class_2339(origin), logs, box, config, height);
		growLeaves(world, rand, origin, leaves, box, config, height, bareTrunkHeight, maxLeafRadius);

		return true;
	}

	private boolean checkForObstructions(class_3746 world, class_2338 origin, int height, int bareTrunkHeight, int radius) {
		class_2338.class_2339 pos = new class_2338.class_2339(origin);

		for (int i = 0; i < bareTrunkHeight; i++) {
			if (!method_16432(world, pos.method_10098(class_2350.field_11036))) {
				return false;
			}
		}

		for (int dY = bareTrunkHeight; dY < height; dY++) {
			for (int dZ = -radius; dZ <= radius; dZ++) {
				for (int dX = -radius; dX <= radius; dX++) {
					pos.method_10103(origin.method_10263() + dX, origin.method_10264() + dY, origin.method_10260() + dZ);

					if (!method_16432(world, pos)) {
						return false;
					}
				}
			}
		}

		return true;
	}

	private void growLeaves(class_3747 world, Random rand, class_2338 origin, Set<class_2338> leaves, class_3341 box, class_4640 config, int height, int bareTrunkHeight, int maxRadius) {
		int radius = 0;
		int radiusTarget = 1;
		boolean topCone = true;

		class_2338.class_2339 pos = new class_2338.class_2339(origin);

		for (int dY = height; dY >= bareTrunkHeight; dY--) {
			for (int dZ = -radius; dZ <= radius; dZ++) {
				for (int dX = -radius; dX <= radius; dX++) {
					int aZ = Math.abs(dZ);
					int aX = Math.abs(dX);

					if (radius > 0 && aZ == radius && aX == radius) {
						// Cull corners
						continue;
					}

					pos.method_10103(origin.method_10263() + dX, origin.method_10264() + dY, origin.method_10260() + dZ);

					if (class_2944.method_16420(world, pos)) {
						int distance = Math.max(aZ + aX, 1);

						PortUtil.setLeavesWithDistance(world, rand, pos, leaves, box, config, distance);
					}
				}
			}

			radius += 1;

			if (radius > radiusTarget) {
				if (topCone) {
					radius = 0;
					radiusTarget = Math.min(2, maxRadius);
					topCone = false;
				} else {
					radius = 1;
					radiusTarget = Math.min(radiusTarget + 1, maxRadius);
				}
			}
		}
	}

	private void growTrunk(class_3747 world, Random rand, class_2338.class_2339 pos, Set<class_2338> logs, class_3341 box, class_4640 config, int height) {
		for (int i = 0; i < height; i++) {
			method_23382(world, rand, pos, logs, box, config);

			pos.method_10098(class_2350.field_11036);
		}
	}

	public int getLeafHeight(Random rand) {
		return rand.nextInt(8) + 24;
	}

	public int getBareTrunkHeight(Random rand) {
		return 1 + rand.nextInt(12);
	}

	public int getMaxLeafRadius(Random rand) {
		return 2 + rand.nextInt(6);
	}
}
