package com.terraformersmc.terrestria.feature.trees.templates;

import com.mojang.datafixers.Dynamic;
import com.terraformersmc.terraform.util.Shapes;
import java.util.Random;
import java.util.Set;
import java.util.function.Function;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2944;
import net.minecraft.class_3341;
import net.minecraft.class_3746;
import net.minecraft.class_3747;
import net.minecraft.class_4640;

public class ConiferTreeFeatureNew extends class_2944<class_4640> {
	public ConiferTreeFeatureNew(Function<Dynamic<?>, ? extends class_4640> function) {
		super(function);
	}

	@Override
	public boolean generate(class_3747 world, Random rand, class_2338 origin, Set<class_2338> logs, Set<class_2338> leaves, class_3341 box, class_4640 config) {
		int height = getLeafHeight(rand);
		int bareTrunkHeight = getBareTrunkHeight(rand);
		int maxLeafRadius = getMaxLeafRadius(rand);
		int leafLayers = getLeafLayers(rand);
		double shrinkAmount = getShrinkAmount();
		int trunkRadius = getMaxTrunkRadius(rand);

		//If the tree doesn't have enough room with it's current height before build limit
		if (origin.method_10264() + height + 1 > 256 || origin.method_10264() < 1) {
			return false;
		}

		//If the ground below the sapling is good enough for a tree
		if (!method_16430(world, origin.method_10074())) {
			return false;
		}

		//If there is room for the tree
		if (!checkForObstructions(world, origin, height, bareTrunkHeight, maxLeafRadius)) {
			return false;
		}

		//Set the block below the trunk to dirt (because vanilla does it)
		method_16427(world, origin.method_10074());
		growTrunk(world, rand, new class_2338.class_2339(origin), logs, box, config, height + bareTrunkHeight, trunkRadius);
		growLeaves(world, rand, new class_2338.class_2339(origin).method_10104(class_2350.field_11036, bareTrunkHeight), leaves, box, config, height, maxLeafRadius, shrinkAmount, leafLayers);

		return true;
	}

	private void growLeaves(class_3747 world, Random rand, class_2338.class_2339 pos, Set<class_2338> leaves, class_3341 box, class_4640 config, int height, int maxRadius, double shrinkAmmount, int layers) {
		int x = pos.method_10263();
		int y = pos.method_10264();
		int z = pos.method_10260();
		int layerHeight = (height / layers);
		int leafLayerHeight = 2 * (height / layers);

		for (int layer = 0; layer < layers; layer++) {
			if (layer == (layers - 1)) { //Limit the top layer to not be too pointy
				leafLayerHeight = layerHeight;
			}
			for (int currentHeight = 0; currentHeight < leafLayerHeight; currentHeight++) {

				pos.method_10103(x, y + (layer * layerHeight) + currentHeight, z);

				Shapes.canopyCircle(pos,
						outerRadius(maxRadius - (layer * shrinkAmmount), currentHeight, leafLayerHeight),
						innerRadius(maxRadius - (layer * shrinkAmmount), currentHeight, layerHeight),
						position -> method_23383(world, rand, pos, leaves, box, config)
				);
			}
		}
	}

	/**
	 * @param maxRadius     the maximum radius for the leaves to generate
	 * @param currentHeight the progress from 0 to the height that the leaves have generated (the x value of the polynomial)
	 * @param height        the target height for the leaf layer
	 * @return the radius of the outside leaf layer at it's current y-height
	 */
	private double outerRadius(double maxRadius, double currentHeight, double height) {
		double x = currentHeight / height;
		// A 3rd-degree polynomial approximating the shape of a Conifer tree. from 0-1
		return maxRadius * ((-0.6 * (x * x * x) + 1.96 * (x * x) - 2.37 * x) + 1);
	}

	/**
	 * @param maxRadius     the maximum radius for the leaves to generate
	 * @param currentHeight the progress from 0 to the height that the leaves have generated (the x value of the polynomial)
	 * @param height        the target height for the leaf layer
	 * @return the radius of the inner leaf layer at it's current y-height
	 */
	private double innerRadius(double maxRadius, double currentHeight, double height) {
		double x = currentHeight / height;
		x = maxRadius * ((-3.24 * (x * x * x) + .25 * (x * x) - 2.98 * x) + 1) - 2.5;
		return x < 0 ? 0 : x;
	}

	public void growTrunk(class_3747 world, Random rand, class_2338.class_2339 pos, Set<class_2338> logs, class_3341 box, class_4640 config, int height, int trunkRadius) {
		for (int i = 0; i < (height * 0.83); i++) {
			method_23382(world, rand, pos, logs, box, config);
			pos.method_10098(class_2350.field_11036);
		}
	}

	private boolean checkForObstructions(class_3746 world, class_2338 origin, int height, int bareTrunkHeight, int radius) {
		class_2338.class_2339 pos = new class_2338.class_2339(origin);

		for (int i = 0; i < bareTrunkHeight; i++) {
			if (!method_16432(world, pos.method_10098(class_2350.field_11036))) {
				return false;
			}
		}

		for (int dY = bareTrunkHeight; dY < height; dY++) {
			for (int dZ = -radius; dZ <= radius; dZ++) {
				for (int dX = -radius; dX <= radius; dX++) {
					pos.method_10103(origin.method_10263() + dX, origin.method_10264() + dY, origin.method_10260() + dZ);

					if (!method_16432(world, pos)) {
						return false;
					}
				}
			}
		}

		return true;
	}

	public int getLeafHeight(Random rand) {
		return rand.nextInt(12) + 32;
	}

	public int getBareTrunkHeight(Random rand) {
		return 8 + rand.nextInt(12);
	}

	public int getMaxLeafRadius(Random rand) {
		return 6 + rand.nextInt(4);
	}

	public int getLeafLayers(Random rand) {
		return rand.nextInt(4) + 4;
	}

	public double getShrinkAmount() {
		return 1.0; //1.0 for 1 block
	}

	public int getMaxTrunkRadius(Random rand) {
		return 1;
	}
}
