package com.terraformersmc.terrestria.feature.trees.templates;

import com.mojang.datafixers.Dynamic;
import com.terraformersmc.terraform.block.SmallLogBlock;
import com.terraformersmc.terraform.util.Shapes;
import com.terraformersmc.terrestria.feature.trees.PortUtil;
import com.terraformersmc.terrestria.feature.trees.components.Branches;
import java.util.*;
import java.util.function.Function;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2944;
import net.minecraft.class_3341;
import net.minecraft.class_3746;
import net.minecraft.class_3747;
import net.minecraft.class_4640;
import net.minecraft.class_4643;

public abstract class JapaneseTreeFeature extends class_2944<class_4640> implements Branches {
	public JapaneseTreeFeature(Function<Dynamic<?>, ? extends class_4640> function) {
		super(function);
	}

	@Override
	public boolean generate(class_3747 world, Random rand, class_2338 origin, Set<class_2338> logs, Set<class_2338> leaves, class_3341 box, class_4640 config) {
		// Total trunk height (8-11)
		int height = rand.nextInt(4) + 8;

		// How much "bare trunk" there will be. (2-4)
		int bareTrunkHeight = 2 + rand.nextInt(3);

		// Maximum leaf radius.
		double maxRadius = 4.0 + 2.5 * rand.nextDouble();

		if (origin.method_10264() + height + 1 > 256 || origin.method_10264() < 1) {
			return false;
		}

		class_2338 below = origin.method_10074();

		if (!method_16430(world, below)) {
			return false;
		}

		if (!checkForObstructions(world, origin, height, bareTrunkHeight, (int) Math.ceil(maxRadius))) {
			return false;
		}

		// Place the log blocks making up the main trunk
		growTrunk(world, rand, new class_2338.class_2339(origin), logs, box, config, height);

		// Grow leaves and branches from the trunk
		class_2338.class_2339 pos = new class_2338.class_2339(origin).method_10104(class_2350.field_11036, bareTrunkHeight);
		growLeaves(world, rand, pos, logs, leaves, box, config, height - bareTrunkHeight, maxRadius);

		// TODO: Better check
		if(!world.method_16358(origin, state -> state.method_11614() instanceof SmallLogBlock)) {
			method_16427(world, below);
		}

		return true;
	}

	private boolean checkForObstructions(class_3746 world, class_2338 origin, int height, int bareTrunkHeight, int maxRadius) {
		class_2338.class_2339 pos = new class_2338.class_2339(origin);

		for (int i = 0; i < bareTrunkHeight; i++) {
			if (!method_16432(world, pos.method_10098(class_2350.field_11036))) {
				return false;
			}
		}

		for (int dY = bareTrunkHeight; dY < height + 1; dY++) {
			int radius = maxRadius * (int) Math.ceil(Math.cos((dY - bareTrunkHeight) * 1.3 / height));

			for (int dZ = -radius; dZ <= radius; dZ++) {
				for (int dX = -radius; dX <= radius; dX++) {
					pos.method_10103(origin.method_10263() + dX, origin.method_10264() + dY, origin.method_10260() + dZ);

					if (!method_16432(world, pos)) {
						return false;
					}
				}
			}
		}

		return true;
	}

	// Grows the center trunk and top leaves of the tree.
	private void growTrunk(class_3747 world, Random rand, class_2338.class_2339 pos, Set<class_2338> logs, class_3341 box, class_4640 config, int height) {
		for (int i = 0; i < height; i++) {
			method_23382(world, rand, pos, logs, box, config);
			pos.method_10098(class_2350.field_11036);
		}
	}

	protected void tryPlaceLeaves(class_3747 world, Random rand, class_2338 pos, Set<class_2338> logs, Set<class_2338> leaves, class_3341 box, class_4643 config) {
		class_2248 reference = config.field_21288.method_23455(rand, pos).method_11614();

		if (reference instanceof SmallLogBlock && world.method_16358(pos, candidate -> candidate.method_11614() == reference)) {
			world.method_16358(pos, candidate ->
					PortUtil.setBlockState(logs, world, pos, candidate.method_11657(SmallLogBlock.HAS_LEAVES, true), box)
			);
		} else {
			method_23383(world, rand, pos, leaves, box, config);
		}
	}

	private void growLeaves(class_3747 world, Random rand, class_2338.class_2339 pos, Set<class_2338> logs, Set<class_2338> leaves, class_3341 box, class_4640 config, int height, double maxRadius) {
		int x = pos.method_10263();
		int y = pos.method_10264();
		int z = pos.method_10260();

		class_2338 origin = pos.method_10062();

		List<class_2350> directionSet = Arrays.asList(class_2350.field_11034, class_2350.field_11039, class_2350.field_11035, class_2350.field_11043);
		Collections.shuffle(directionSet);
		Iterator<class_2350> directions = directionSet.iterator();

		for (int layer = 0; layer < 2; layer++) {
			int chance = 2 + layer * 5;
			double radius = maxRadius * Math.cos(layer * 1.3 / height);
			double innerRadius = Math.max(radius, 2) - 2;

			for (int i = 0; i < 2; i++) {
				pos.method_10103(x, y + layer, z);

				class_2350 direction = directions.next();
				placeBranch(world, rand, pos, logs, leaves, box, config, direction, (int) Math.ceil(innerRadius));

				pos.method_10098(direction);

				tryPlaceLeaves(world, rand, pos, logs, leaves, box, config);
			}

			pos.method_10103(x, y + layer, z);
			Shapes.canopyCircle(pos, radius, innerRadius,
					(position) -> {
						if (rand.nextInt(chance) == 0 && position.method_10263() != origin.method_10263() && position.method_10260() != origin.method_10260()) {
							return;
						}

						tryPlaceLeaves(world, rand, position, logs, leaves, box, config);
					}
			);
		}

		for (int layer = 2; layer < height + 1; layer++) {
			double radius = maxRadius * Math.cos(layer * 1.3 / height);
			double innerRadius = maxRadius * Math.max(Math.cos(layer * (Math.PI / 4.0) / height + (Math.PI / 3.0)), 0);

			if (layer < height) {
				class_2350 direction;

				switch (rand.nextInt(4)) {
					case 0:
						direction = class_2350.field_11034;
						break;
					case 1:
						direction = class_2350.field_11039;
						break;
					case 2:
						direction = class_2350.field_11035;
						break;
					default:
						direction = class_2350.field_11043;
						break;
				}

				pos.method_10103(x, y + layer, z);
				placeBranch(world, rand, pos, logs, leaves, box, config, direction, (int) Math.ceil(innerRadius));
			}

			pos.method_10103(x, y + layer, z);
			Shapes.canopyCircle(pos, radius, innerRadius,
					(position) -> tryPlaceLeaves(world, rand, pos, logs, leaves, box, config)
			);
		}
	}
}
