package com.terraformersmc.terrestria.init.helpers;

import com.terraformersmc.terrestria.feature.TreeDefinition;
import com.terraformersmc.terraform.block.*;
import com.terraformersmc.terrestria.Terrestria;
import net.fabricmc.fabric.api.block.FabricBlockSettings;
import net.fabricmc.fabric.api.registry.FlammableBlockRegistry;
import net.fabricmc.fabric.api.registry.FuelRegistry;
import net.minecraft.block.*;
import net.minecraft.class_2246;
import net.minecraft.class_2248;
import net.minecraft.class_2349;
import net.minecraft.class_2354;
import net.minecraft.class_2397;
import net.minecraft.class_2410;
import net.minecraft.class_2482;
import net.minecraft.class_2960;
import java.util.function.Supplier;

public class WoodBlocks {
	public class_2248 log;
	public class_2248 wood;
	public class_2248 leaves;
	public class_2248 planks;
	public class_2482 slab;
	public TerraformStairsBlock stairs;
	public class_2354 fence;
	public class_2349 fenceGate;
	public TerraformDoorBlock door;
	public TerraformButtonBlock button;
	public TerraformPressurePlateBlock pressurePlate;
	public TerraformSignBlock sign;
	public TerraformWallSignBlock wallSign;
	public TerraformTrapdoorBlock trapdoor;
	public class_2248 strippedLog;
	public class_2248 strippedWood;
	private String name;
	private WoodColors colors;

	public WoodBlocks() {}

	public static WoodBlocks register(String name, WoodColors colors, FlammableBlockRegistry registry, boolean useExtendedLeaves) {
		WoodBlocks blocks = registerManufactured(name, colors, registry);

		blocks.log = TerrestriaRegistry.register(name + "_log", new StrippableLogBlock(() -> blocks.strippedLog, colors.planks, FabricBlockSettings.copy(class_2246.field_10431).materialColor(colors.bark).build()));
		blocks.wood = TerrestriaRegistry.register(name + "_wood", new StrippableLogBlock(() -> blocks.strippedWood, colors.bark, FabricBlockSettings.copy(class_2246.field_10431).materialColor(colors.bark).build()));
		if (useExtendedLeaves) {
			blocks.leaves = TerrestriaRegistry.register(name + "_leaves", new ExtendedLeavesBlock(FabricBlockSettings.copy(class_2246.field_10503).materialColor(colors.leaves).build()));
		} else {
			blocks.leaves = TerrestriaRegistry.register(name + "_leaves", new class_2397(FabricBlockSettings.copy(class_2246.field_10503).materialColor(colors.leaves).build()));
		}
		blocks.strippedLog = TerrestriaRegistry.register("stripped_" + name + "_log", new class_2410(colors.planks, FabricBlockSettings.copy(class_2246.field_10431).materialColor(colors.planks).build()));
		blocks.strippedWood = TerrestriaRegistry.register("stripped_" + name + "_wood", new class_2410(colors.planks, FabricBlockSettings.copy(class_2246.field_10431).materialColor(colors.planks).build()));

		blocks.addTreeFireInfo(registry);

		return blocks;
	}

	public static WoodBlocks register(String name, WoodColors colors, FlammableBlockRegistry registry) {
		return register(name, colors, registry, false);
	}

	public static WoodBlocks register(String name, WoodColors colors, FlammableBlockRegistry registry, LogSize size) {
		if (size.equals(LogSize.SMALL)) {
			WoodBlocks blocks = registerManufactured(name, colors, registry);

			blocks.leaves = TerrestriaRegistry.register(name + "_leaves", new TransparentLeavesBlock(FabricBlockSettings.copy(class_2246.field_10503).materialColor(colors.leaves).build()));

			blocks.log = TerrestriaRegistry.register(name + "_log", new SmallLogBlock(blocks.leaves, () -> blocks.strippedLog, FabricBlockSettings.copy(class_2246.field_10431).materialColor(colors.bark).build()));
			blocks.wood = blocks.log; // no need for a wood type

			blocks.strippedLog = TerrestriaRegistry.register("stripped_" + name + "_log", new SmallLogBlock(blocks.leaves, null, FabricBlockSettings.copy(class_2246.field_10431).materialColor(colors.planks).build()));
			blocks.strippedWood = blocks.strippedLog; // wno need for a stripped wood type

			blocks.addTreeFireInfo(registry);

			return blocks;
		} else {
			return register(name, colors, registry, false);
		}
	}

	public static WoodBlocks registerManufactured(String name, WoodColors colors, FlammableBlockRegistry registry) {
		WoodBlocks blocks = new WoodBlocks();
		blocks.name = name;
		blocks.colors = colors;

		blocks.planks = TerrestriaRegistry.register(name + "_planks", new class_2248(FabricBlockSettings.copy(class_2246.field_10161).materialColor(colors.planks).build()));
		blocks.slab = TerrestriaRegistry.register(name + "_slab", new class_2482(FabricBlockSettings.copy(class_2246.field_10119).materialColor(colors.planks).build()));
		blocks.stairs = TerrestriaRegistry.register(name + "_stairs", new TerraformStairsBlock(blocks.planks, FabricBlockSettings.copy(class_2246.field_10563).materialColor(colors.planks).build()));
		blocks.fence = TerrestriaRegistry.register(name + "_fence", new class_2354(FabricBlockSettings.copy(class_2246.field_10620).materialColor(colors.planks).build()));
		blocks.fenceGate = TerrestriaRegistry.register(name + "_fence_gate", new class_2349(FabricBlockSettings.copy(class_2246.field_10188).materialColor(colors.planks).build()));
		blocks.door = TerrestriaRegistry.register(name + "_door", new TerraformDoorBlock(FabricBlockSettings.copy(class_2246.field_10188).materialColor(colors.planks).build()));
		blocks.button = TerrestriaRegistry.register(name + "_button", new TerraformButtonBlock(FabricBlockSettings.copy(class_2246.field_10057).materialColor(colors.planks).build()));
		blocks.pressurePlate = TerrestriaRegistry.register(name + "_pressure_plate", new TerraformPressurePlateBlock(FabricBlockSettings.copy(class_2246.field_10484).materialColor(colors.planks).build()));
		blocks.trapdoor = TerrestriaRegistry.register(name + "_trapdoor", new TerraformTrapdoorBlock(FabricBlockSettings.copy(class_2246.field_10137).materialColor(colors.planks).build()));

		class_2960 signTexture = new class_2960(Terrestria.MOD_ID, "entity/signs/" + name);

		blocks.sign = TerrestriaRegistry.register(name + "_sign", new TerraformSignBlock(signTexture, FabricBlockSettings.copy(class_2246.field_10121).materialColor(colors.planks).build()));
		blocks.wallSign = TerrestriaRegistry.register(name + "_wall_sign", new TerraformWallSignBlock(signTexture, FabricBlockSettings.copy(class_2246.field_10121).materialColor(colors.planks).build()));

		blocks.addManufacturedFireInfo(registry);

		FuelRegistry.INSTANCE.add(blocks.fence, 300);
		FuelRegistry.INSTANCE.add(blocks.fenceGate, 300);

		return blocks;
	}

	public QuarterLogBlock registerQuarterLog(Supplier<class_2248> stripped, FlammableBlockRegistry registry) {
		QuarterLogBlock quarterLog = TerrestriaRegistry.register(name + "_quarter_log", new QuarterLogBlock(stripped, colors.planks, class_2248.class_2251.method_9630(log)));

		registry.add(quarterLog, 5, 5);

		return quarterLog;
	}

	public QuarterLogBlock registerStrippedQuarterLog(FlammableBlockRegistry registry) {
		QuarterLogBlock quarterLog = TerrestriaRegistry.register("stripped_" + name + "_quarter_log", new QuarterLogBlock(null, colors.planks, class_2248.class_2251.method_9630(strippedLog)));

		registry.add(quarterLog, 5, 5);

		return quarterLog;
	}

	public void addTreeFireInfo(FlammableBlockRegistry registry) {
		registry.add(log, 5, 5);
		registry.add(strippedLog, 5, 5);

		if (wood != log) {
			registry.add(wood, 5, 5);
		}

		if (strippedWood != strippedLog) {
			registry.add(strippedWood, 5, 5);
		}

		registry.add(leaves, 30, 60);
	}

	public void addManufacturedFireInfo(FlammableBlockRegistry registry) {
		registry.add(planks, 5, 20);
		registry.add(slab, 5, 20);
		registry.add(stairs, 5, 20);
		registry.add(fence, 5, 20);
		registry.add(fenceGate, 5, 20);
	}

	public TreeDefinition.Mega getMegaDefinition(class_2248 quarterLog) {
		return new TreeDefinition.Mega(this.log.method_9564(), this.leaves.method_9564(), quarterLog.method_9564(), this.wood.method_9564());
	}

	public enum LogSize {
		LARGE("large"),
		SMALL("small");

		private final String name;

		LogSize(String name) {
			this.name = name;
		}

		public String getName() {
			return this.name;
		}
	}
}
